#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>
#include <iomanip>

using namespace std;

namespace xyzformat {

typedef struct atomdata {
  string symbol;
  double x, y, z;
} Atom;

int readxyz(const string& filename, Atom *(&atomList), double (*metricTensor)[3]) 
{

  ifstream in(filename.c_str());
  if(in.fail()) // or if(!in)
  {
    cerr << "Error opening file " << filename << endl;
    exit(EXIT_FAILURE);
  }

  int natoms;
  in >> natoms;

  if (in.fail()) {
    cerr << "Bad file format" << endl;
    exit(EXIT_FAILURE);
  } else if (natoms < 1) {
    cerr << "Wrong number of atoms " << endl;
    exit(EXIT_FAILURE);
  }

  try {
    // Atom *atomList = (Atom *) malloc (natoms*sizeof(Atom));
    atomList = new Atom[natoms];
  } catch (bad_alloc){
    cerr << "Bad dynamic memory allocation" << endl;
    exit(EXIT_FAILURE);
  }

  int line = 0;
  for (; line<natoms && !in.eof(); line++) {
    Atom temp;
    in >> temp.symbol;
    in >> temp.x;
    in >> temp.y;
    in >> temp.z;

    if (in.fail()) {
      cerr << "Bad format at line " << line << endl;
      break;
    }

    atomList[line].symbol = temp.symbol;
    atomList[line].x = temp.x;
    atomList[line].y = temp.y;
    atomList[line].z = temp.z;
  }

  if (line < natoms) {
    cerr << "Read only " << line << "lines instead of " 
      << natoms << endl;
  }

  for (int i=0; i<3;i++) {
    in >> metricTensor[i][0] >> metricTensor[i][1] >>metricTensor[i][2];
  }

  return line;
}

void writexyz(const string& filename, int natoms, const Atom *atomList, const double (*metricTensor)[3]) 
{

  ofstream out(filename.c_str());
  if(out.fail()) // or if(!out)
  {
    cerr << "Error opening file "<<filename << endl;
    exit(EXIT_FAILURE);
  }

  out << natoms << endl << endl;

  int w1 = 4, w2 = 15; // field width for symbol and coordinates respectively

  out << scientific << setprecision(8);
  for (int line = 0; line<natoms; line++) {
    out << left << setw(w1) << atomList[line].symbol;
    out << right << setw(w2) << atomList[line].x << "  " 
                 << setw(w2) << atomList[line].y << "  "
                 << setw(w2) << atomList[line].z << endl;
  }

  out << endl;

  for (int i = 0; i < 3; i++ ) { // metrictensor: three elements per line
    out << setw(5) << " "
        << metricTensor[i][0] << "   " 
        << metricTensor[i][1] << "   " 
        << metricTensor[i][2] << endl;
  }    
}

} // xyzformat namespace

using namespace xyzformat;

int main()
{

  Atom *atomList;
  double metricTensor[3][3];

  string filename = "xyzinput.dat";
  // cout << "Enter filename "<< endl;
  // cin >> filename;

  int natoms = readxyz(filename, atomList, metricTensor);
  cout << "read " << natoms << " atoms entries" << endl;
  cerr << atomList[natoms-1].symbol << " " 
      << atomList[natoms-1].x << " "
      << atomList[natoms-1].y << " "
      << atomList[natoms-1].z << endl;

  string outfile = "copy.xyz";

  writexyz(outfile, natoms, atomList, metricTensor);

  return 0;
}
