#include <iostream>
#include <limits>
#include <cstdlib>
#include <climits>
#include "lfrng.h"

using namespace std;
using namespace LFRNG;


double runMean(unsigned long niter, rng &r) {
  // Windows rand() produces only 16 bit integers
  // 10^5 draws should cause all 32 bits to be filled up
  for (unsigned long i=0; i<100000; i++)
    r.draw();

  double sum = 0.0;
  for (unsigned long i=0; i<niter; i++) {
    sum += r.draw();
  }
  return sum/niter;
}



void test_init(rng &r) {
  const unsigned iter = 10000;
  cout << "Testing init() method:" << endl;
  cout << "Mean before init():  " << runMean(iter, r) << endl;
  r.init();
  cout << "Mean after init():  " << runMean(iter, r) << endl;
}



int main () {
  // parameters for testing purposes
  unsigned long test=0, ntests = 5;
  unsigned long niter = 10000;

  cout << "Testing Lagged Fibonacci RNG" << endl;
  cout << "Theoric mean is " 
    << UINT_MAX/2.0 << endl << endl;
  
  do {
      // direct instances of LFRNGs
      rng srgen(55), lrgen(98);
      rng *dynamicRng = new rng(60);


      cout << "Mean for ShortRng:  " << runMean(niter, srgen) << endl;

      cout << "Mean for MediumRng: " << runMean(niter, *dynamicRng) << endl;

      cout << "Mean for LongRng:   " << runMean(niter, lrgen) << endl;

      cout << endl 
        << " --- test :" << test << " --- " << endl
        << endl;

      test++;
      delete dynamicRng;

  } while (test < ntests);

  // rng whatrng();
  //
  // will cause a compile time error like:
  //
  // test_lfrng.cpp:57: error: no matching function for call to ‘LFRNG::rng::rng()’
  // lfrng.h:21: note: candidates are: LFRNG::rng::rng(unsigned int)
  // lfrng.h:8: note:                 LFRNG::rng::rng(const LFRNG::rng&)

  rng srgen(55), mrgen(60);

  // the following assignments will result in double free() system calls since
  // lrgen.hstr content is overwritten with mrgen.hstr value and two destructors
  // for different objects will try to delete the same memory block twice
  //
  // mrgen = srgen;    
  // rng lrgen = mrgen;

  // initializing from integer constant
  rng crng = 22;
  cout << "Draw from integer constant initialization: " << crng.draw() << endl;

  rng *dynamicRng = new rng(70);
  test_init(*dynamicRng);
  delete dynamicRng;

  return 0;
}
