#include <iostream>
#include <limits>
#include <cstdlib>
#include "lfrng.h"

using namespace std;
using namespace LFRNG;


double runMean(unsigned long niter, rng &r) {
  // Windows rand() produces only 16 bit integers
  // 10^5 draws should cause all 32 bits to be filled up
  for (unsigned long i=0; i<100000; i++)
    r();

  double sum = 0.0;
  for (unsigned long i=0; i<niter; i++) {
    sum += r();
  }
  return sum/niter;
}



int main () {
  // parameters for testing purposes
  unsigned long test=0, ntests = 5;
  unsigned long niter = 10000;

  cout << "Testing Lagged Fibonacci RNG" << endl;
  cout << "Max_len() = " << rng::max_len() << endl;
  cout << "Theoric mean is " 
    << numeric_limits<unsigned>::max()/2.0 << endl << endl;
  
  do {
      // direct instances of LFRNGs
      rng srgen(55), lrgen(98);
      rng *dynamicRng = new rng(60);


      cout << "Mean for ShortRng:  " << runMean(niter, srgen) << endl;

      cout << "Mean for MediumRng: " << runMean(niter, *dynamicRng) << endl;

      cout << "Mean for LongRng:   " << runMean(niter, lrgen) << endl;

      cout << endl 
        << " --- test :" << test << " --- " << endl
        << endl;

      test++;
      delete dynamicRng;

  } while (test < ntests);

  // rng whatrng();
  //
  // will cause a compile time error like:
  //
  // test_lfrng.cpp:57: error: no matching function for call to ‘LFRNG::rng::rng()’
  // lfrng.h:21: note: candidates are: LFRNG::rng::rng(unsigned int)
  // lfrng.h:8: note:                 LFRNG::rng::rng(const LFRNG::rng&)

  rng srgen(55), mrgen(60);

  // the following assignments will result in a compile time error 
  // mrgen = srgen;
  // rng lrgen = mrgen;
  //
  // lfrng.h:29: error: ‘LFRNG::rng& LFRNG::rng::operator=(const LFRNG::rng&)’ is protected
  // test_lfrng.cpp:63: error: within this context
  // lfrng.h:28: error: ‘LFRNG::rng::rng(const LFRNG::rng&)’ is protected
  // test_lfrng.cpp:64: error: within this context

  // rng crng = 22;
  //
  // initializing from integer constant causes a compile time error like:
  // test_lfrng.cpp:71: error: conversion from ‘int’ to non-scalar type ‘LFRNG::rng’ requested

  return 0;
}
