#include <iostream>
#include "lfrng.h"

using namespace std;
using namespace LFRNG;

class evenrng : protected rng {
  public:
    explicit evenrng(unsigned n=98) : rng(n) {};
    evenrng(unsigned n, const unsigned *a) : rng(n,a) {}
    unsigned operator() () {
      return rng::draw() & ~1U; // equivalent to (rng::draw()/2)*2
    }
};

class oddrng : protected rng {
  public:
    explicit oddrng(unsigned n=98) : rng(n) {};
    oddrng(unsigned n, const unsigned *a) : rng(n,a) {}
    unsigned operator() () {
      return rng::draw() | 1U; // equivalent to (rng::draw()/2)*2+1
    }
};

double runMean(unsigned long niter, coin &r) {
  // Windows rand() produces only 16 bit integers
  // 10^5 draws should cause all 32 bits to be filled up
  for (unsigned long i=0; i<100000; i++)
    r();

  double sum = 0.0;
  for (unsigned long i=0; i<niter; i++) {
    sum += r();
  }
  return sum/niter;
}



int main () {

  coin penny;
  evenrng even(98);
  oddrng  odd(50);

  cout << "Mean for coin class:  " << runMean(100000, penny) << endl;
  cout << "Even coin: " << even() << " Odd coin: " << odd() << endl;

  return 0;
}
