#ifndef POSITION_H
#define POSITION_H

#include <cmath>
#include "vect.h"

template<class T=double> struct position : public vect<T> {
  protected:
    position(vect<T> r) : vect<T>(r) {}

  public:
    position() {}
    position(double a, double b, double c) : vect<T>(a, b, c) {}
    
    position operator+() const { return *this; }
    position operator-() const { return this->vect<T>::operator-(); }
    
    position& operator+= (position r) { this->vect<T>::operator+=(r); return *this; }
    position& operator-= (position r) { this->vect<T>::operator-=(r); return *this; }
    position& operator*= (T s) { this->vect<T>::operator*=(s); return *this; }
    position& operator/= (T s) { this->vect<T>::operator/=(s); return *this; }

    position operator+ (position r) const { return this->vect<T>::operator+(r); }
    position operator- (position r) const { return this->vect<T>::operator-(r); }
    T operator* (position r) const { return this->vect<T>::operator*(r); }
    position operator* (T s) const { return this->vect<T>::operator*(s); }
    template<class F> friend position<F> operator* (F s, position<F> r);
    position operator/ (T s) const { return this->vect<T>::operator/(s); }
    bool operator== (position r) const { return this->vect<T>::operator==(r); }
    bool operator!= (position r) const { return this->vect<T>::operator!=(r); }
    
    template<class F> friend std::ostream & operator<<(std::ostream &s, const position<F> &r);

};

template<class F> inline position<F> operator* (F s, position<F> v) {
  v *= s;
  return v;
}

template<class F> inline F abs(position<F> v) {
  return abs(vect<F>(v));
}

template<class F> std::ostream & operator<<(std::ostream &s, const position<F> &r)
{ return s << static_cast< vect<F> >(r); }

#endif
