#ifndef VELOCITY_H
#define VELOCITY_H

#include <cmath>
#include "vect.h"

template<class T=double> struct velocity : public vect<T> {
  protected:
    velocity(vect<T> r) : vect<T>(r) {}

  public:
    velocity() {}
    velocity(double a, double b, double c) : vect<T>(a, b, c) {}
    
    velocity operator+() const { return *this; }
    velocity operator-() const { return this->vect<T>::operator-(); }
    
    velocity& operator+= (velocity r) { this->vect<T>::operator+=(r); return *this; }
    velocity& operator-= (velocity r) { this->vect<T>::operator-=(r); return *this; }
    velocity& operator*= (T s) { this->vect<T>::operator*=(s); return *this; }
    velocity& operator/= (T s) { this->vect<T>::operator/=(s); return *this; }

    velocity operator+ (velocity r) const { return this->vect<T>::operator+(r); }
    velocity operator- (velocity r) const { return this->vect<T>::operator-(r); }
    T operator* (velocity r) const { return this->vect<T>::operator*(r); }
    velocity operator* (T s) const { return this->vect<T>::operator*(s); }
    template<class F> friend velocity<F> operator* (F s, velocity<F> r);
    velocity operator/ (T s) const { return this->vect<T>::operator/(s); }
    bool operator== (velocity r) const { return this->vect<T>::operator==(r); }
    bool operator!= (velocity r) const { return this->vect<T>::operator!=(r); }
    
    template<class F> friend std::ostream & operator<<(std::ostream &s, const velocity<F> &r);

};

template<class F> inline velocity<F> operator* (F s, velocity<F> v) {
  v *= s;
  return v;
}

template<class F> inline F abs(velocity<F> v) {
  return abs(vect<F>(v));
}

template<class F> std::ostream & operator<<(std::ostream &s, const velocity<F> &r)
{ return s << static_cast< vect<F> >(r); }

#endif
