#ifndef PWLINEAR_H
#define PWLINEAR_H

#include <cstring>
#include <stdexcept>
#include "interpolator.h"

// piecewise linear interpolation policy
template<class T> struct pwlinear {
protected:
    static T line(const T p[][2], T x) {
        return (x - p[0][0]) * (p[1][1] - p[0][1]) / (p[1][0] - p[0][0])
               + p[0][1];             
    }
public:
    static T interpolate(int n, const T table[][2], T x) {
        int i;  // we'll need the counter value AFTER the loop
        
        if (n < 2)
            throw std::invalid_argument("Two or more data points required");

        if (x < table[0][0])
            throw std::out_of_range("interpolator");
        
        for (i = 1; i < n; ++i)           // linear serch is inefficient!
            if (x < table[i][0]) break;   // this is only a naive example
        
        if (i == n && x != table[n-1][0])
            throw std::out_of_range("interpolator");
        
        return line(table+i-1, x);        // linear interpolation
    }
};

// piecewise linear interpolation policy
// with insane extrapolation for out of range arguments
// privately inherits line() from pwlinear<T>
template<class T> struct pwlinear_ext : private pwlinear<T> {
    static T interpolate(int n, const T table[][2], T x) {
        int i;  // we'll need the counter value AFTER the loop
        
        if (n < 2) throw std::invalid_argument("Two or more data points required");

        if (x < table[0][0])
            return line(table, x); // linear extrapolation
        
        for (i = 1; i < n; ++i)           // linear serch is inefficient!
            if (x < table[i][0]) break;   // this is only a naive example
        
        if (i == n && x != table[n-1][0]) 
            return line(table+n-1, x);    // linear extrapolation
        
        return line(table+i-1, x);        // linear interpolation
    }
};

#endif
