#ifndef BINGO_H
#define BINGO_H

#include "lfrng.h"
#include <vector>

namespace LFRNG {

class bingo : protected rng {
    static const unsigned numbers = 90U;

    std::vector<unsigned> cage;
    std::vector<unsigned> drawn_numbers;
    
    void initCage() {
        cage.reserve(numbers);
        for(int i=0; i<numbers; i++) cage.push_back(i+1);
    };
public:
    explicit bingo(unsigned n=98) : rng(n) {
            initCage();
        }
    bingo(unsigned n, const unsigned *a) : rng(n,a) {
            initCage();
        }

    unsigned drawn() const { return drawn_numbers.size(); }

    unsigned left() const { return cage.size(); }

    bool empty() const { return cage.empty(); }

    unsigned drawn(unsigned i) const {
        unsigned t;
        try {
            t = drawn_numbers.at(i-1);
        } catch (std::out_of_range) {
            return 0;
        }
        return t;
    }

    unsigned operator() () {
         if (!cage.empty()) {
             unsigned i = rng::draw()%cage.size();
             unsigned t = cage[i];
             cage[i] = cage.back();  // shorthand for cage[cage.size()-1]
             cage.pop_back();
             drawn_numbers.push_back(t);
             return t;
         } else
           return 0;
    }
};

} // NAMESPACE LFRNG


#endif