#include <iostream>
#include <iomanip>
#include <boost/accumulators/accumulators.hpp>
#include <boost/accumulators/statistics/stats.hpp>
#include <boost/accumulators/statistics/mean.hpp>
#include <boost/accumulators/statistics/min.hpp>
#include <boost/accumulators/statistics/max.hpp>
#include <boost/accumulators/statistics/moment.hpp>
#include <boost/accumulators/statistics/density.hpp>
#include "frng.h"

using std::cout;
using std::endl;
using std::setw;
using namespace boost::accumulators;
using namespace FPRNG;

const int nsamples = 10000;

void test_fr(frng *g) {
    typedef boost::iterator_range<std::vector<std::pair<double, double> >::iterator > histtype;

    const int nstart = 2000; // Number of samples used to define the bins width
    accumulator_set<double, stats<tag::density, tag::moment<2>, tag::mean, 
        tag::min, tag::max > > acc(tag::density::num_bins = 10, 
        tag::density::cache_size = nstart);

  // Windows rand() produces only 16 bit integers
  // 10^5 draws should cause all 32 bits to be filled up
	for(int i = 0; i< 100000; ++i)
	    (*g)();
	
	for (int i = 0; i< nsamples; ++i) {
	    double r = (*g)();
        acc(r);
	}

    histtype hist = density(acc);	

    cout << "Average = " << mean(acc) << " Variance = " << moment<2>(acc) <<  endl;
    cout << "Min and max values = " << min(acc) << ", " << max(acc) <<  endl;
    cout << endl;

    for ( int i = 0; i < hist.size(); i++)
        cout << setw(11) << hist[i].first << setw(11) << hist[i].second << endl;
}

int main() {
	furng uniform;
	ferng exponential;
	fsurng scaled(-1.0, 2.0);
	fnrng normal;
	ftrianglerng triangle;
	
    cout << "Using " << nsamples << " samples:" << endl;
	test_fr(&uniform);
	test_fr(&exponential);
	test_fr(&scaled);
	test_fr(&normal);
	test_fr(&triangle);
	
	return 0;
}
